import java.util.ArrayList;

public class ClosetManager {
	
	/*
	 * This is a list of lists of clothing. For this example, we can think of 
	 * it in simpler terms as a list of closets with each closet holding some
	 * clothes.
	 */
	private ArrayList<ArrayList<Clothing>> closets = 
			new ArrayList<ArrayList<Clothing>>();
	
	
	/*
	 * This method adds ArrayList<Clothing> newCloset to the list of lists.
	 */
	public void addCloset(ArrayList<Clothing> newCloset) {
		closets.add(newCloset);
	}
	
	/*
	 * This method inserts a Clothing recentPurchase into the list of clothing
	 * at index closetNumber in the list of lists of Clothing.
	 */
	public void insertClothing(Clothing recentPurchase, int closetNumber) {
		closets.get(closetNumber).add(recentPurchase);
	}
		
	/*
	 * Returns a reference copy of our list of lists of clothing.
	 */
	public ArrayList<ArrayList<Clothing>> getReferenceCopy(){
		return closets;
	}
	
	/*
	 * This shallow copy creates a new list of lists of clothes which points to
	 * the same lists of clothes as the original. In other words, it is a new 
	 * list of closets, but it references the same closets.
	 */
	public ArrayList<ArrayList<Clothing>> getShallowCopy(){
		ArrayList<ArrayList<Clothing>> copy = 
				new ArrayList<ArrayList<Clothing>>();
		
		for (ArrayList<Clothing> singleCloset : closets) {
			copy.add(singleCloset);
		}
		
		return copy;
	}
	
	/*
	 * This shallow copy creates a new list of lists of clothes as well as new 
	 * lists of clothes to go on that list. However, each new list of clothes 
	 * contains only references to the original articles of clothing. It's like 
	 * we built a new house with new closets, but each new closet contains only 
	 * a list of references to the same clothing in the old house. This is 
	 * where reality bends a little bit.
	 */
	public ArrayList<ArrayList<Clothing>> getAlsoShallowCopy(){
		ArrayList<ArrayList<Clothing>> copy = 
				new ArrayList<ArrayList<Clothing>>();
		
		for (ArrayList<Clothing> originalCloset : closets) {
			ArrayList<Clothing> closetCopy = new ArrayList<Clothing>();
			
			// What if we moved the line "copy.add(closetCopy);" to here?
			// Would the code still work? Why?
			
			for (Clothing article : originalCloset) {
				closetCopy.add(article);
			}
			
			copy.add(closetCopy);
		}
		
		return copy;
	}
	
	/*
	 * This is a deep which will create a new copy of every object all the way
	 * down to the String instance variables in each clothing object 
	 */
	public ArrayList<ArrayList<Clothing>> getDeepCopy(){
		ArrayList<ArrayList<Clothing>> copy = 
				new ArrayList<ArrayList<Clothing>>();
		
		for (ArrayList<Clothing> originalCloset : closets) {
			ArrayList<Clothing> closetCopy = new ArrayList<Clothing>();
			
			for (Clothing article : originalCloset) {
				closetCopy.add(new Clothing(article));
			}
			
			copy.add(closetCopy);
		}
		
		return copy;
	}
	
	
	/*
	 *  toString that returns a list of closets and how the clothing inside 
	 *  them is feeling.
	 */
	public String toString() {
		String output = new String();
		
		for (ArrayList<Clothing> singleCloset : closets) {
			int closetNumber = closets.indexOf(singleCloset);
			output += "In closet number " + closetNumber + ":\n";
			
			for (Clothing article : singleCloset) {
				output += article + "\n";
			}
		}
		
		return output;
	}
}
